@@ -32,14 +32,16 @@ optional traits:
 
 import logging
 from dataclasses import dataclass, field, fields
+from functools import cache
 from typing import Any, get_args
 
 from roborock.data.containers import HomeData, HomeDataProduct, RoborockBase
 from roborock.data.v1.v1_code_mappings import RoborockDockTypeCode
-from roborock.devices.cache import Cache
+from roborock.devices.cache import DeviceCache
 from roborock.devices.traits import Trait
-from roborock.devices.v1_rpc_channel import V1RpcChannel
 from roborock.map.map_parser import MapParserConfig
+from roborock.protocols.v1_protocol import V1RpcChannel
+from roborock.web_api import UserWebApiClient
 
 from .child_lock import ChildLockTrait
 from .clean_summary import CleanSummaryTrait
@@ -56,6 +58,7 @@ from .map_content import MapContentTrait
 from .maps import MapsTrait
 from .network_info import NetworkInfoTrait
 from .rooms import RoomsTrait
+from .routines import RoutinesTrait
 from .smart_wash_params import SmartWashParamsTrait
 from .status import StatusTrait
 from .valley_electricity_timer import ValleyElectricityTimerTrait
@@ -65,26 +68,27 @@ from .wash_towel_mode import WashTowelModeTrait
 _LOGGER = logging.getLogger(__name__)
 
 __all__ = [
-    "create",
     "PropertiesApi",
-    "StatusTrait",
-    "DoNotDisturbTrait",
-    "CleanSummaryTrait",
-    "SoundVolumeTrait",
-    "MapsTrait",
-    "MapContentTrait",
-    "ConsumableTrait",
-    "HomeTrait",
-    "DeviceFeaturesTrait",
-    "CommandTrait",
-    "ChildLockTrait",
-    "FlowLedStatusTrait",
-    "LedStatusTrait",
-    "ValleyElectricityTimerTrait",
-    "DustCollectionModeTrait",
-    "WashTowelModeTrait",
-    "SmartWashParamsTrait",
-    "NetworkInfoTrait",
+    "child_lock",
+    "clean_summary",
+    "common",
+    "consumeable",
+    "device_features",
+    "do_not_disturb",
+    "dust_collection_mode",
+    "flow_led_status",
+    "home",
+    "led_status",
+    "map_content",
+    "maps",
+    "network_info",
+    "rooms",
+    "routines",
+    "smart_wash_params",
+    "status",
+    "valley_electricity_timer",
+    "volume",
+    "wash_towel_mode",
 ]
 
 
@@ -108,6 +112,7 @@ class PropertiesApi(Trait):
     home: HomeTrait
     device_features: DeviceFeaturesTrait
     network_info: NetworkInfoTrait
+    routines: RoutinesTrait
 
     # Optional features that may not be supported on all devices
     child_lock: ChildLockTrait | None = None
@@ -126,7 +131,8 @@ class PropertiesApi(Trait):
         rpc_channel: V1RpcChannel,
         mqtt_rpc_channel: V1RpcChannel,
         map_rpc_channel: V1RpcChannel,
-        cache: Cache,
+        web_api: UserWebApiClient,
+        device_cache: DeviceCache,
         map_parser_config: MapParserConfig | None = None,
     ) -> None:
         """Initialize the V1TraitProps."""
@@ -134,16 +140,18 @@ class PropertiesApi(Trait):
         self._rpc_channel = rpc_channel
         self._mqtt_rpc_channel = mqtt_rpc_channel
         self._map_rpc_channel = map_rpc_channel
-        self._cache = cache
+        self._web_api = web_api
+        self._device_cache = device_cache
 
         self.status = StatusTrait(product)
         self.consumables = ConsumableTrait()
         self.rooms = RoomsTrait(home_data)
         self.maps = MapsTrait(self.status)
         self.map_content = MapContentTrait(map_parser_config)
-        self.home = HomeTrait(self.status, self.maps, self.map_content, self.rooms, cache)
-        self.device_features = DeviceFeaturesTrait(product.product_nickname, cache)
-        self.network_info = NetworkInfoTrait(device_uid, cache)
+        self.home = HomeTrait(self.status, self.maps, self.map_content, self.rooms, self._device_cache)
+        self.device_features = DeviceFeaturesTrait(product.product_nickname, self._device_cache)
+        self.network_info = NetworkInfoTrait(device_uid, self._device_cache)
+        self.routines = RoutinesTrait(device_uid, web_api)
 
         # Dynamically create any traits that need to be populated
         for item in fields(self):
@@ -232,7 +240,7 @@ class PropertiesApi(Trait):
 
     async def _get_cached_trait_data(self, name: str) -> Any:
         """Get the dock type from the status trait or cache."""
-        cache_data = await self._cache.get()
+        cache_data = await self._device_cache.get()
         if cache_data.trait_data is None:
             cache_data.trait_data = {}
         _LOGGER.debug("Cached trait data: %s", cache_data.trait_data)
@@ -240,12 +248,12 @@ class PropertiesApi(Trait):
 
     async def _set_cached_trait_data(self, name: str, value: Any) -> None:
         """Set trait-specific cached data."""
-        cache_data = await self._cache.get()
+        cache_data = await self._device_cache.get()
         if cache_data.trait_data is None:
             cache_data.trait_data = {}
         cache_data.trait_data[name] = value
         _LOGGER.debug("Updating cached trait data: %s", cache_data.trait_data)
-        await self._cache.set(cache_data)
+        await self._device_cache.set(cache_data)
 
     def as_dict(self) -> dict[str, Any]:
         """Return the trait data as a dictionary."""
@@ -267,7 +275,8 @@ def create(
     rpc_channel: V1RpcChannel,
     mqtt_rpc_channel: V1RpcChannel,
     map_rpc_channel: V1RpcChannel,
-    cache: Cache,
+    web_api: UserWebApiClient,
+    device_cache: DeviceCache,
     map_parser_config: MapParserConfig | None = None,
 ) -> PropertiesApi:
     """Create traits for V1 devices."""
@@ -278,6 +287,7 @@ def create(
         rpc_channel,
         mqtt_rpc_channel,
         map_rpc_channel,
-        cache,
+        web_api,
+        device_cache,
         map_parser_config,
     )
